@@ -8,10 +8,13 @@ from bdata import bdata
 from scipy.optimize import curve_fit
 from tqdm import tqdm
 from bfit.fitting.global_bdata_fitter import global_bdata_fitter
+from bfit.fitting.leastsquares import LeastSquares
+from iminuit import Minuit
+import inspect
 
 # ========================================================================== #
-def fit_bdata(data,fn,omit=None,rebin=None,shared=None,hist_select='',
-              xlims=None,asym_mode='c',fixed=None,**kwargs):
+def fit_bdata(data, fn, omit=None, rebin=None, shared=None, hist_select='',
+              xlims=None, asym_mode='c', fixed=None, minimizer='migrad+minos', **kwargs):
     """
         Fit combined asymetry from bdata.
     
@@ -53,16 +56,21 @@ def fit_bdata(data,fn,omit=None,rebin=None,shared=None,hist_select='',
                         parameters in order presented, with the fixed 
                         parameters omitted. Can be a list of lists with one list 
                         for each run.
+                        
+        minimizer       string. One of "migrad", "trf", "dogbox"
         
-        kwargs:         keyword arguments for curve_fit. See curve_fit docs. 
+        kwargs:         keyword arguments for curve_fit/minuit. 
+                        See curve_fit/iminuit docs. 
         
-        Returns: (par,std,cov,chi,gchi)
-            par: array of best fit parameters
-            std: array of best fit errors
-            cov: 2D array, covariance matrix
-            chi: array of chisquare of each fit
-            gchi:global chisquared of fits
+        Returns: (par, std_l, std_h, cov, chi, gchi)
+            par:    array of best fit parameters
+            std_l:  array of lower best fit errors
+            std_h:  array of upper best fit errors
+            cov:    2D array, covariance matrix
+            chi:    array of chisquare of each fit
+            gchi:   global chisquared of fits
     """
+    
     try:
         ndata = len(data)
     except TypeError:
@@ -86,13 +94,13 @@ def fit_bdata(data,fn,omit=None,rebin=None,shared=None,hist_select='',
                                
     # get shared
     if shared is None:
-        shared = np.zeros(npar,dtype=bool)
+        shared = np.zeros(npar, dtype=bool)
 
     # get omit
     if omit is None:
         omit = ['']*ndata
     elif len(omit) < ndata:
-        omit = np.concatenate((omit,['']*(ndata-len(omit))))
+        omit = np.concatenate((omit, ['']*(ndata-len(omit))))
         
     # get rebin
     if rebin is None:
@@ -100,18 +108,29 @@ def fit_bdata(data,fn,omit=None,rebin=None,shared=None,hist_select='',
     elif type(rebin) is int:
         rebin = np.ones(ndata)*rebin
     elif len(rebin) < ndata:
-        rebin = np.concatenate((rebin,np.ones(ndata-len(rebin))))
+        rebin = np.concatenate((rebin, np.ones(ndata-len(rebin))))
     rebin = np.asarray(rebin).astype(int)
         
     # fit globally -----------------------------------------------------------
     if any(shared) and ndata>1:
-        print('Running shared parameter fitting... ',end='',flush=True)
-        g = global_bdata_fitter(data,fn,shared,xlims,
-                                asym_mode=asym_mode,rebin=rebin,fixed=fixed)
-        g.fit(**kwargs)
-        gchi,chis = g.get_chi() # returns global chi, individual chi
-        pars,covs,stds = g.get_par()
-        print('done.',flush=True)
+        print('Running shared parameter fitting... ', flush=True)
+        do_minos = 'minos' in minimizer
+        minimizer = 'migrad' if 'migrad' in minimizer else minimizer
+            
+        g = global_bdata_fitter(data = data, 
+                                fn = fn, 
+                                xlims = xlims,
+                                shared = shared, 
+                                asym_mode = asym_mode, 
+                                rebin = rebin, 
+                                fixed = fixed,
+                                )
+                                
+        g.fit(minimizer=minimizer, do_minos=do_minos, **kwargs)
+        gchi, chis = g.get_chi() # returns global chi, individual chi squared
+        pars, stds_l, stds_h, covs = g.get_par()
+        
+        print('done.', flush=True)
         
     # fit runs individually --------------------------------------------------
     else:
@@ -149,16 +168,17 @@ def fit_bdata(data,fn,omit=None,rebin=None,shared=None,hist_select='',
         pars = []
         covs = []
         chis = []
-        stds = []
+        stds_l = []
+        stds_h = []
         gchi = 0.
         dof = 0.
         
-        iter_obj = tqdm(zip(data,fn,omit,rebin,p0,bounds,xlims,fixed),
-                        total=ndata,desc='Independent Fitting')
-        for d,f,om,re,p,b,xl,fix in iter_obj:
+        iter_obj = tqdm(zip(data, fn, omit, rebin, p0, bounds, xlims, fixed),
+                        total=ndata, desc='Independent Fitting')
+        for d, f, om, re, p, b, xl, fix in iter_obj:
             
             # get data for chisq calculations
-            x,y,dy = _get_asym(d,asym_mode,rebin=re,omit=om)
+            x,y,dy = _get_asym(d, asym_mode, rebin=re, omit=om)
             
             # get x limits
             if xl is None:  
@@ -176,20 +196,47 @@ def fit_bdata(data,fn,omit=None,rebin=None,shared=None,hist_select='',
             # trivial case: all parameters fixed
             if all(fix):
                 lenp = len(p)
-                s = np.full((lenp,lenp),np.nan)
-                c = np.sum(np.square((y-f(x,*p))/dy))/len(y)
+                c = np.full((lenp,lenp),np.nan)
+                s = np.diag(c)
+                ch = np.sum(np.square((y-f(x,*p))/dy))/len(y)
                 
             # fit with free parameters
             else:            
                 kwargs['p0'] = p
                 kwargs['bounds'] = b
-                p,s,c = _fit_single(d,f,om,re,hist_select,xlim=xl,
-                                    asym_mode=asym_mode,fixed=fix,**kwargs)
+                p, c, sl, sh, ch, m = _fit_single(d, f, om, re, hist_select, xlim=xl,
+                                    asym_mode=asym_mode, fixed=fix, 
+                                    minimizer=minimizer, **kwargs)
+                                    
+                # check minuit validity
+                if m is not None: 
+                    
+                    
+                    if not all((m.fmin.is_valid, 
+                                m.fmin.has_valid_parameters,
+                                not m.fmin.hesse_failed,
+                                m.fmin.has_accurate_covar,
+                                m.fmin.has_covariance,
+                                m.fmin.has_posdef_covar,
+                                not m.fmin.has_made_posdef_covar,
+                                not m.fmin.has_reached_call_limit,
+                                not m.fmin.is_above_max_edm,                                
+                                )):
+                                    
+                        msg = ('====== %d.%d ======\n' % (d.year, d.run),
+                               str(m.fmin), '\n',
+                               str(m.params), '\n',
+                               )
+                               
+                        iter_obj.write(''.join(msg))
+                    
+                    
             # outputs
             pars.append(p)
-            covs.append(s)
-            chis.append(c)
-            stds.append(np.diag(s)**0.5)
+            covs.append(c)
+            stds_l.append(sl)
+            stds_h.append(sh)
+            chis.append(ch)
             
             # get global chi             
             gchi += np.sum(np.square((y-f(x,*p))/dy))
@@ -198,21 +245,23 @@ def fit_bdata(data,fn,omit=None,rebin=None,shared=None,hist_select='',
         
     pars = np.asarray(pars)
     covs = np.asarray(covs)
-    stds = np.asarray(stds)
+    stds_l = np.asarray(stds_l)
+    stds_h = np.asarray(stds_h)
     chis = np.asarray(chis)
     
     # single data set fitting
     if ndata == 1:
         pars = pars[0]
-        stds = stds[0]
+        stds_l = stds_l[0]
+        stds_h = stds_h[0]
         covs = covs[0]
         chis = chis[0]
     
-    return(pars,stds,covs,chis,gchi)
+    return(pars, stds_l, stds_h, covs, chis, gchi)
 
 # =========================================================================== #
 def _fit_single(data,fn,omit='',rebin=1,hist_select='',xlim=None,asym_mode='c',
-               fixed=None,**kwargs):
+               fixed=None, minimizer='migrad+minos', **kwargs):
     """
         Fit combined asymetry from bdata.
     
@@ -243,6 +292,8 @@ def _fit_single(data,fn,omit='',rebin=1,hist_select='',xlim=None,asym_mode='c',
                         fixed to p0 value (same length as p0). Returns best 
                         parameters in order presented, with the fixed 
                         parameters omitted.
+                        
+        minimizer       string. One of "migrad", "trf", "dogbox"
         
         kwargs:         keyword arguments for curve_fit. See curve_fit docs. 
         
@@ -276,6 +327,71 @@ def _fit_single(data,fn,omit='',rebin=1,hist_select='',xlim=None,asym_mode='c',
                                'Define p0 to resolve.')
         kwargs['p0'] = np.ones(nargs)
     
+    # Fit the function
+    if "migrad" in minimizer:
+        par, cov, stdl, stdh, chi, m = _fit_single_minuit(fn, x, y, dy, fixed, 
+                                                          'minos' in minimizer, 
+                                                          **kwargs)
+    elif minimizer in ('trf', 'dogbox'):
+        par, cov, stdl, stdh, chi = _fit_single_curve_fit(fn, x, y, dy, fixed,  
+                                                          minimizer, **kwargs)
+        m = None
+    
+    return (par, cov, stdl, stdh, chi, m)
+    
+# =========================================================================== #
+def _fit_single_minuit(fn, x, y, dy, fixed, do_minos=True, **kwargs):
+    """
+        Fit data with minuit minimizer
+    """
+    
+    # make least squares function 
+    ls = LeastSquares(fn, x, y, dy)
+    
+    # set up minuit inputs
+    bounds = np.array(kwargs['bounds']).T
+    
+    kwargs_minuit = {'start':kwargs['p0'],
+                     'limit':bounds,
+                     'fix':fixed,
+                     'print_level':kwargs.get('print_level',0),
+                     'errordef':1,
+                     }
+    
+    name = kwargs.get('name',None)
+    if name is None:    
+        name = inspect.getfullargspec(fn).args
+        if 'self' in name:                  name.remove('self')
+        if len(name) == len(kwargs['p0']):  kwargs_minuit['name'] = name
+    else:
+        kwargs_minuit['name'] = name
+
+    m = Minuit.from_array_func(ls, **kwargs_minuit)
+    m.migrad()
+    
+    if do_minos:
+        m.minos()
+        lower, upper = m.np_merrors()
+    else:
+        m.hesse()
+        err = m.np_errors()
+        lower, upper = (err, err)
+    
+    # get parameters
+    par = m.np_values()
+    cov = m.np_covariance()
+    
+    dof = len(y) - len(kwargs['p0'])
+    chi = m.fval/dof
+    
+    return (par, cov, lower, upper, chi, m)
+
+# =========================================================================== #
+def _fit_single_curve_fit(fn, x, y, dy, fixed, minimizer, **kwargs):
+    """
+        Fit data with curve_fit minimizers
+    """
+    
     # fixed parameters
     did_fixed = False
     if fixed is not None and any(fixed):
@@ -285,29 +401,24 @@ def _fit_single(data,fn,omit='',rebin=1,hist_select='',xlim=None,asym_mode='c',
         p0 = np.copy(kwargs['p0'])
         npar = len(p0)
         
-        # dumb case: all values fixed: 
-        if all(fixed):
-            cov = np.full((npar,npar),np.nan)
-            chi = np.sum(np.square((y-fn(x,*p0))/dy))/len(y)
-            return (p0,cov,chi)
-        
         # prep inputs
         fixed = np.asarray(fixed)
         if 'bounds' in kwargs:  bounds = kwargs['bounds']
         else:                   bounds = None
         
         # get fixed version
-        fn,kwargs['p0'],bounds = _get_fixed_values(fixed,fn,kwargs['p0'],bounds)
+        fn,kwargs['p0'], bounds = _get_fixed_values(fixed, fn, kwargs['p0'], bounds)
         
         # modify fiting inputs
         if bounds is not None:  kwargs['bounds'] = bounds
         
-    # Fit the function 
-    par,cov = curve_fit(fn,x,y,sigma=dy,absolute_sigma=True,**kwargs)
-    dof = len(y)-len(kwargs['p0'])
+    # do the fit
+    par,cov = curve_fit(fn, x, y, sigma=dy, absolute_sigma=True, 
+                        method=minimizer, **kwargs)
+    dof = len(y) - len(kwargs['p0'])
     
     # get chisquared
-    chi = np.sum(np.square((y-fn(x,*par))/dy))/dof
+    chi = np.sum(np.square((y-fn(x, *par)) / dy)) / dof
     
     # inflate parameters with fixed values 
     if did_fixed:
@@ -324,8 +435,11 @@ def _fit_single(data,fn,omit='',rebin=1,hist_select='',xlim=None,asym_mode='c',
         c_inflated[nfixed_flat] = np.concatenate(cov)
         cov = c_inflated.reshape(npar,-1)
     
-    return (par,cov,chi)
+    # get errors
+    std = np.diag(cov)**0.5
     
+    return (par, cov, std, std, chi)
+
 # =========================================================================== #
 def _get_asym(data,asym_mode,**asym_kwargs):
     """
